# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import httpx

from ...._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ...._utils import maybe_transform, async_maybe_transform
from ...._compat import cached_property
from ...._resource import SyncAPIResource, AsyncAPIResource
from ...._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...._base_client import make_request_options
from ....types.ai.fine_tuning import job_create_params
from ....types.ai.fine_tuning.fine_tuning_job import FineTuningJob
from ....types.ai.fine_tuning.job_list_response import JobListResponse

__all__ = ["JobsResource", "AsyncJobsResource"]


class JobsResource(SyncAPIResource):
    """Customize LLMs for your unique needs"""

    @cached_property
    def with_raw_response(self) -> JobsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return JobsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> JobsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return JobsResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        model: str,
        training_file: str,
        hyperparameters: job_create_params.Hyperparameters | Omit = omit,
        suffix: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> FineTuningJob:
        """
        Create a new fine tuning job.

        Args:
          model: The base model that is being fine-tuned.

          training_file: The storage bucket or object used for training.

          hyperparameters: The hyperparameters used for the fine-tuning job.

          suffix: Optional suffix to append to the fine tuned model's name.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/ai/fine_tuning/jobs",
            body=maybe_transform(
                {
                    "model": model,
                    "training_file": training_file,
                    "hyperparameters": hyperparameters,
                    "suffix": suffix,
                },
                job_create_params.JobCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=FineTuningJob,
        )

    def retrieve(
        self,
        job_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> FineTuningJob:
        """
        Retrieve a fine tuning job by `job_id`.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not job_id:
            raise ValueError(f"Expected a non-empty value for `job_id` but received {job_id!r}")
        return self._get(
            f"/ai/fine_tuning/jobs/{job_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=FineTuningJob,
        )

    def list(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> JobListResponse:
        """Retrieve a list of all fine tuning jobs created by the user."""
        return self._get(
            "/ai/fine_tuning/jobs",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=JobListResponse,
        )

    def cancel(
        self,
        job_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> FineTuningJob:
        """
        Cancel a fine tuning job.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not job_id:
            raise ValueError(f"Expected a non-empty value for `job_id` but received {job_id!r}")
        return self._post(
            f"/ai/fine_tuning/jobs/{job_id}/cancel",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=FineTuningJob,
        )


class AsyncJobsResource(AsyncAPIResource):
    """Customize LLMs for your unique needs"""

    @cached_property
    def with_raw_response(self) -> AsyncJobsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return AsyncJobsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncJobsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return AsyncJobsResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        model: str,
        training_file: str,
        hyperparameters: job_create_params.Hyperparameters | Omit = omit,
        suffix: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> FineTuningJob:
        """
        Create a new fine tuning job.

        Args:
          model: The base model that is being fine-tuned.

          training_file: The storage bucket or object used for training.

          hyperparameters: The hyperparameters used for the fine-tuning job.

          suffix: Optional suffix to append to the fine tuned model's name.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/ai/fine_tuning/jobs",
            body=await async_maybe_transform(
                {
                    "model": model,
                    "training_file": training_file,
                    "hyperparameters": hyperparameters,
                    "suffix": suffix,
                },
                job_create_params.JobCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=FineTuningJob,
        )

    async def retrieve(
        self,
        job_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> FineTuningJob:
        """
        Retrieve a fine tuning job by `job_id`.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not job_id:
            raise ValueError(f"Expected a non-empty value for `job_id` but received {job_id!r}")
        return await self._get(
            f"/ai/fine_tuning/jobs/{job_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=FineTuningJob,
        )

    async def list(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> JobListResponse:
        """Retrieve a list of all fine tuning jobs created by the user."""
        return await self._get(
            "/ai/fine_tuning/jobs",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=JobListResponse,
        )

    async def cancel(
        self,
        job_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> FineTuningJob:
        """
        Cancel a fine tuning job.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not job_id:
            raise ValueError(f"Expected a non-empty value for `job_id` but received {job_id!r}")
        return await self._post(
            f"/ai/fine_tuning/jobs/{job_id}/cancel",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=FineTuningJob,
        )


class JobsResourceWithRawResponse:
    def __init__(self, jobs: JobsResource) -> None:
        self._jobs = jobs

        self.create = to_raw_response_wrapper(
            jobs.create,
        )
        self.retrieve = to_raw_response_wrapper(
            jobs.retrieve,
        )
        self.list = to_raw_response_wrapper(
            jobs.list,
        )
        self.cancel = to_raw_response_wrapper(
            jobs.cancel,
        )


class AsyncJobsResourceWithRawResponse:
    def __init__(self, jobs: AsyncJobsResource) -> None:
        self._jobs = jobs

        self.create = async_to_raw_response_wrapper(
            jobs.create,
        )
        self.retrieve = async_to_raw_response_wrapper(
            jobs.retrieve,
        )
        self.list = async_to_raw_response_wrapper(
            jobs.list,
        )
        self.cancel = async_to_raw_response_wrapper(
            jobs.cancel,
        )


class JobsResourceWithStreamingResponse:
    def __init__(self, jobs: JobsResource) -> None:
        self._jobs = jobs

        self.create = to_streamed_response_wrapper(
            jobs.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            jobs.retrieve,
        )
        self.list = to_streamed_response_wrapper(
            jobs.list,
        )
        self.cancel = to_streamed_response_wrapper(
            jobs.cancel,
        )


class AsyncJobsResourceWithStreamingResponse:
    def __init__(self, jobs: AsyncJobsResource) -> None:
        self._jobs = jobs

        self.create = async_to_streamed_response_wrapper(
            jobs.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            jobs.retrieve,
        )
        self.list = async_to_streamed_response_wrapper(
            jobs.list,
        )
        self.cancel = async_to_streamed_response_wrapper(
            jobs.cancel,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .cursor import Cursor as Cursor
from .feature import Feature as Feature
from .metadata import Metadata as Metadata
from .api_error import APIError as APIError
from .short_code import ShortCode as ShortCode
from .hosted_number import HostedNumber as HostedNumber
from .sim_card_status import SimCardStatus as SimCardStatus
from .simple_sim_card import SimpleSimCard as SimpleSimCard
from .cost_information import CostInformation as CostInformation
from .netapps_location import NetappsLocation as NetappsLocation
from .room_participant import RoomParticipant as RoomParticipant
from .region_information import RegionInformation as RegionInformation
from .rime_voice_settings import RimeVoiceSettings as RimeVoiceSettings
from .azure_voice_settings import AzureVoiceSettings as AzureVoiceSettings
from .porting_order_status import PortingOrderStatus as PortingOrderStatus
from .messaging_feature_set import MessagingFeatureSet as MessagingFeatureSet
from .number_health_metrics import NumberHealthMetrics as NumberHealthMetrics
from .minimax_voice_settings import MinimaxVoiceSettings as MinimaxVoiceSettings
from .whatsapp_template_data import WhatsappTemplateData as WhatsappTemplateData
from .inbound_message_payload import InboundMessagePayload as InboundMessagePayload
from .resemble_voice_settings import ResembleVoiceSettings as ResembleVoiceSettings
from .connection_jitter_buffer import ConnectionJitterBuffer as ConnectionJitterBuffer
from .doc_reqs_requirement_type import DocReqsRequirementType as DocReqsRequirementType
from .messaging_pagination_meta import MessagingPaginationMeta as MessagingPaginationMeta
from .connections_pagination_meta import ConnectionsPaginationMeta as ConnectionsPaginationMeta
from .messaging_hosted_number_order import MessagingHostedNumberOrder as MessagingHostedNumberOrder
from .porting_orders_exception_type import PortingOrdersExceptionType as PortingOrdersExceptionType
from .phone_numbers_job_phone_number import PhoneNumbersJobPhoneNumber as PhoneNumbersJobPhoneNumber
from .available_phone_numbers_metadata import AvailablePhoneNumbersMetadata as AvailablePhoneNumbersMetadata
from .connection_noise_suppression_details import ConnectionNoiseSuppressionDetails as ConnectionNoiseSuppressionDetails
from .phone_number_with_messaging_settings import PhoneNumberWithMessagingSettings as PhoneNumberWithMessagingSettings
from .sub_number_order_regulatory_requirement_with_value import (
    SubNumberOrderRegulatoryRequirementWithValue as SubNumberOrderRegulatoryRequirementWithValue,
)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import httpx
import pytest
from respx import MockRouter

from telnyx import Telnyx, AsyncTelnyx
from tests.utils import assert_matches_type
from telnyx.types import (
    VoiceCloneData,
    VoiceCloneCreateResponse,
    VoiceCloneUpdateResponse,
    VoiceCloneCreateFromUploadResponse,
)
from telnyx._response import (
    BinaryAPIResponse,
    AsyncBinaryAPIResponse,
    StreamedBinaryAPIResponse,
    AsyncStreamedBinaryAPIResponse,
)
from telnyx.pagination import SyncDefaultFlatPagination, AsyncDefaultFlatPagination

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestVoiceClones:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_create(self, client: Telnyx) -> None:
        voice_clone = client.voice_clones.create(
            gender="male",
            language="en",
            name="clone-narrator",
            voice_design_id="550e8400-e29b-41d4-a716-446655440000",
        )
        assert_matches_type(VoiceCloneCreateResponse, voice_clone, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_create(self, client: Telnyx) -> None:
        response = client.voice_clones.with_raw_response.create(
            gender="male",
            language="en",
            name="clone-narrator",
            voice_design_id="550e8400-e29b-41d4-a716-446655440000",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        voice_clone = response.parse()
        assert_matches_type(VoiceCloneCreateResponse, voice_clone, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_create(self, client: Telnyx) -> None:
        with client.voice_clones.with_streaming_response.create(
            gender="male",
            language="en",
            name="clone-narrator",
            voice_design_id="550e8400-e29b-41d4-a716-446655440000",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            voice_clone = response.parse()
            assert_matches_type(VoiceCloneCreateResponse, voice_clone, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_update(self, client: Telnyx) -> None:
        voice_clone = client.voice_clones.update(
            id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            name="updated-clone",
        )
        assert_matches_type(VoiceCloneUpdateResponse, voice_clone, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_update_with_all_params(self, client: Telnyx) -> None:
        voice_clone = client.voice_clones.update(
            id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            name="updated-clone",
            gender="male",
            language="language",
        )
        assert_matches_type(VoiceCloneUpdateResponse, voice_clone, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_update(self, client: Telnyx) -> None:
        response = client.voice_clones.with_raw_response.update(
            id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            name="updated-clone",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        voice_clone = response.parse()
        assert_matches_type(VoiceCloneUpdateResponse, voice_clone, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_update(self, client: Telnyx) -> None:
        with client.voice_clones.with_streaming_response.update(
            id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            name="updated-clone",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            voice_clone = response.parse()
            assert_matches_type(VoiceCloneUpdateResponse, voice_clone, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_path_params_update(self, client: Telnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.voice_clones.with_raw_response.update(
                id="",
                name="updated-clone",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_list(self, client: Telnyx) -> None:
        voice_clone = client.voice_clones.list()
        assert_matches_type(SyncDefaultFlatPagination[VoiceCloneData], voice_clone, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_list_with_all_params(self, client: Telnyx) -> None:
        voice_clone = client.voice_clones.list(
            filter_name="filter[name]",
            page_number=1,
            page_size=1,
            sort="name",
        )
        assert_matches_type(SyncDefaultFlatPagination[VoiceCloneData], voice_clone, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_list(self, client: Telnyx) -> None:
        response = client.voice_clones.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        voice_clone = response.parse()
        assert_matches_type(SyncDefaultFlatPagination[VoiceCloneData], voice_clone, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_list(self, client: Telnyx) -> None:
        with client.voice_clones.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            voice_clone = response.parse()
            assert_matches_type(SyncDefaultFlatPagination[VoiceCloneData], voice_clone, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_delete(self, client: Telnyx) -> None:
        voice_clone = client.voice_clones.delete(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert voice_clone is None

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_delete(self, client: Telnyx) -> None:
        response = client.voice_clones.with_raw_response.delete(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        voice_clone = response.parse()
        assert voice_clone is None

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_delete(self, client: Telnyx) -> None:
        with client.voice_clones.with_streaming_response.delete(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            voice_clone = response.parse()
            assert voice_clone is None

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_path_params_delete(self, client: Telnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.voice_clones.with_raw_response.delete(
                "",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_create_from_upload(self, client: Telnyx) -> None:
        voice_clone = client.voice_clones.create_from_upload(
            audio_file=b"Example data",
            language="lkf-Lz1vLbBu-9uDh-9AHaOS2D-Cbf",
            name="name",
        )
        assert_matches_type(VoiceCloneCreateFromUploadResponse, voice_clone, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_create_from_upload_with_all_params(self, client: Telnyx) -> None:
        voice_clone = client.voice_clones.create_from_upload(
            audio_file=b"Example data",
            language="lkf-Lz1vLbBu-9uDh-9AHaOS2D-Cbf",
            name="name",
            gender="male",
            label="label",
            ref_text="ref_text",
        )
        assert_matches_type(VoiceCloneCreateFromUploadResponse, voice_clone, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_create_from_upload(self, client: Telnyx) -> None:
        response = client.voice_clones.with_raw_response.create_from_upload(
            audio_file=b"Example data",
            language="lkf-Lz1vLbBu-9uDh-9AHaOS2D-Cbf",
            name="name",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        voice_clone = response.parse()
        assert_matches_type(VoiceCloneCreateFromUploadResponse, voice_clone, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_create_from_upload(self, client: Telnyx) -> None:
        with client.voice_clones.with_streaming_response.create_from_upload(
            audio_file=b"Example data",
            language="lkf-Lz1vLbBu-9uDh-9AHaOS2D-Cbf",
            name="name",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            voice_clone = response.parse()
            assert_matches_type(VoiceCloneCreateFromUploadResponse, voice_clone, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    @pytest.mark.respx(base_url=base_url)
    def test_method_download_sample(self, client: Telnyx, respx_mock: MockRouter) -> None:
        respx_mock.get("/voice_clones/182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e/sample").mock(
            return_value=httpx.Response(200, json={"foo": "bar"})
        )
        voice_clone = client.voice_clones.download_sample(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert voice_clone.is_closed
        assert voice_clone.json() == {"foo": "bar"}
        assert cast(Any, voice_clone.is_closed) is True
        assert isinstance(voice_clone, BinaryAPIResponse)

    @parametrize
    @pytest.mark.respx(base_url=base_url)
    def test_raw_response_download_sample(self, client: Telnyx, respx_mock: MockRouter) -> None:
        respx_mock.get("/voice_clones/182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e/sample").mock(
            return_value=httpx.Response(200, json={"foo": "bar"})
        )

        voice_clone = client.voice_clones.with_raw_response.download_sample(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )

        assert voice_clone.is_closed is True
        assert voice_clone.http_request.headers.get("X-Stainless-Lang") == "python"
        assert voice_clone.json() == {"foo": "bar"}
        assert isinstance(voice_clone, BinaryAPIResponse)

    @parametrize
    @pytest.mark.respx(base_url=base_url)
    def test_streaming_response_download_sample(self, client: Telnyx, respx_mock: MockRouter) -> None:
        respx_mock.get("/voice_clones/182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e/sample").mock(
            return_value=httpx.Response(200, json={"foo": "bar"})
        )
        with client.voice_clones.with_streaming_response.download_sample(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        ) as voice_clone:
            assert not voice_clone.is_closed
            assert voice_clone.http_request.headers.get("X-Stainless-Lang") == "python"

            assert voice_clone.json() == {"foo": "bar"}
            assert cast(Any, voice_clone.is_closed) is True
            assert isinstance(voice_clone, StreamedBinaryAPIResponse)

        assert cast(Any, voice_clone.is_closed) is True

    @parametrize
    @pytest.mark.respx(base_url=base_url)
    def test_path_params_download_sample(self, client: Telnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.voice_clones.with_raw_response.download_sample(
                "",
            )


class TestAsyncVoiceClones:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_create(self, async_client: AsyncTelnyx) -> None:
        voice_clone = await async_client.voice_clones.create(
            gender="male",
            language="en",
            name="clone-narrator",
            voice_design_id="550e8400-e29b-41d4-a716-446655440000",
        )
        assert_matches_type(VoiceCloneCreateResponse, voice_clone, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_create(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.voice_clones.with_raw_response.create(
            gender="male",
            language="en",
            name="clone-narrator",
            voice_design_id="550e8400-e29b-41d4-a716-446655440000",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        voice_clone = await response.parse()
        assert_matches_type(VoiceCloneCreateResponse, voice_clone, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_create(self, async_client: AsyncTelnyx) -> None:
        async with async_client.voice_clones.with_streaming_response.create(
            gender="male",
            language="en",
            name="clone-narrator",
            voice_design_id="550e8400-e29b-41d4-a716-446655440000",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            voice_clone = await response.parse()
            assert_matches_type(VoiceCloneCreateResponse, voice_clone, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_update(self, async_client: AsyncTelnyx) -> None:
        voice_clone = await async_client.voice_clones.update(
            id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            name="updated-clone",
        )
        assert_matches_type(VoiceCloneUpdateResponse, voice_clone, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_update_with_all_params(self, async_client: AsyncTelnyx) -> None:
        voice_clone = await async_client.voice_clones.update(
            id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            name="updated-clone",
            gender="male",
            language="language",
        )
        assert_matches_type(VoiceCloneUpdateResponse, voice_clone, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_update(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.voice_clones.with_raw_response.update(
            id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            name="updated-clone",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        voice_clone = await response.parse()
        assert_matches_type(VoiceCloneUpdateResponse, voice_clone, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_update(self, async_client: AsyncTelnyx) -> None:
        async with async_client.voice_clones.with_streaming_response.update(
            id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            name="updated-clone",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            voice_clone = await response.parse()
            assert_matches_type(VoiceCloneUpdateResponse, voice_clone, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_path_params_update(self, async_client: AsyncTelnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.voice_clones.with_raw_response.update(
                id="",
                name="updated-clone",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_list(self, async_client: AsyncTelnyx) -> None:
        voice_clone = await async_client.voice_clones.list()
        assert_matches_type(AsyncDefaultFlatPagination[VoiceCloneData], voice_clone, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_list_with_all_params(self, async_client: AsyncTelnyx) -> None:
        voice_clone = await async_client.voice_clones.list(
            filter_name="filter[name]",
            page_number=1,
            page_size=1,
            sort="name",
        )
        assert_matches_type(AsyncDefaultFlatPagination[VoiceCloneData], voice_clone, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_list(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.voice_clones.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        voice_clone = await response.parse()
        assert_matches_type(AsyncDefaultFlatPagination[VoiceCloneData], voice_clone, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncTelnyx) -> None:
        async with async_client.voice_clones.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            voice_clone = await response.parse()
            assert_matches_type(AsyncDefaultFlatPagination[VoiceCloneData], voice_clone, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_delete(self, async_client: AsyncTelnyx) -> None:
        voice_clone = await async_client.voice_clones.delete(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert voice_clone is None

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_delete(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.voice_clones.with_raw_response.delete(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        voice_clone = await response.parse()
        assert voice_clone is None

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_delete(self, async_client: AsyncTelnyx) -> None:
        async with async_client.voice_clones.with_streaming_response.delete(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            voice_clone = await response.parse()
            assert voice_clone is None

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_path_params_delete(self, async_client: AsyncTelnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.voice_clones.with_raw_response.delete(
                "",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_create_from_upload(self, async_client: AsyncTelnyx) -> None:
        voice_clone = await async_client.voice_clones.create_from_upload(
            audio_file=b"Example data",
            language="lkf-Lz1vLbBu-9uDh-9AHaOS2D-Cbf",
            name="name",
        )
        assert_matches_type(VoiceCloneCreateFromUploadResponse, voice_clone, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_create_from_upload_with_all_params(self, async_client: AsyncTelnyx) -> None:
        voice_clone = await async_client.voice_clones.create_from_upload(
            audio_file=b"Example data",
            language="lkf-Lz1vLbBu-9uDh-9AHaOS2D-Cbf",
            name="name",
            gender="male",
            label="label",
            ref_text="ref_text",
        )
        assert_matches_type(VoiceCloneCreateFromUploadResponse, voice_clone, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_create_from_upload(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.voice_clones.with_raw_response.create_from_upload(
            audio_file=b"Example data",
            language="lkf-Lz1vLbBu-9uDh-9AHaOS2D-Cbf",
            name="name",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        voice_clone = await response.parse()
        assert_matches_type(VoiceCloneCreateFromUploadResponse, voice_clone, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_create_from_upload(self, async_client: AsyncTelnyx) -> None:
        async with async_client.voice_clones.with_streaming_response.create_from_upload(
            audio_file=b"Example data",
            language="lkf-Lz1vLbBu-9uDh-9AHaOS2D-Cbf",
            name="name",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            voice_clone = await response.parse()
            assert_matches_type(VoiceCloneCreateFromUploadResponse, voice_clone, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    @pytest.mark.respx(base_url=base_url)
    async def test_method_download_sample(self, async_client: AsyncTelnyx, respx_mock: MockRouter) -> None:
        respx_mock.get("/voice_clones/182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e/sample").mock(
            return_value=httpx.Response(200, json={"foo": "bar"})
        )
        voice_clone = await async_client.voice_clones.download_sample(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert voice_clone.is_closed
        assert await voice_clone.json() == {"foo": "bar"}
        assert cast(Any, voice_clone.is_closed) is True
        assert isinstance(voice_clone, AsyncBinaryAPIResponse)

    @parametrize
    @pytest.mark.respx(base_url=base_url)
    async def test_raw_response_download_sample(self, async_client: AsyncTelnyx, respx_mock: MockRouter) -> None:
        respx_mock.get("/voice_clones/182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e/sample").mock(
            return_value=httpx.Response(200, json={"foo": "bar"})
        )

        voice_clone = await async_client.voice_clones.with_raw_response.download_sample(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )

        assert voice_clone.is_closed is True
        assert voice_clone.http_request.headers.get("X-Stainless-Lang") == "python"
        assert await voice_clone.json() == {"foo": "bar"}
        assert isinstance(voice_clone, AsyncBinaryAPIResponse)

    @parametrize
    @pytest.mark.respx(base_url=base_url)
    async def test_streaming_response_download_sample(self, async_client: AsyncTelnyx, respx_mock: MockRouter) -> None:
        respx_mock.get("/voice_clones/182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e/sample").mock(
            return_value=httpx.Response(200, json={"foo": "bar"})
        )
        async with async_client.voice_clones.with_streaming_response.download_sample(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        ) as voice_clone:
            assert not voice_clone.is_closed
            assert voice_clone.http_request.headers.get("X-Stainless-Lang") == "python"

            assert await voice_clone.json() == {"foo": "bar"}
            assert cast(Any, voice_clone.is_closed) is True
            assert isinstance(voice_clone, AsyncStreamedBinaryAPIResponse)

        assert cast(Any, voice_clone.is_closed) is True

    @parametrize
    @pytest.mark.respx(base_url=base_url)
    async def test_path_params_download_sample(self, async_client: AsyncTelnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.voice_clones.with_raw_response.download_sample(
                "",
            )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import TypedDict

__all__ = ["RunTriggerParams"]


class RunTriggerParams(TypedDict, total=False):
    destination_version_id: str
    """Optional assistant version ID to use for this test run.

    If provided, the version must exist or a 400 error will be returned. If not
    provided, test will run on main version
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List, Optional
from datetime import datetime

from .._models import BaseModel

__all__ = ["SessionAnalysisRetrieveResponse", "Cost", "Meta"]


class Cost(BaseModel):
    currency: str
    """ISO 4217 currency code."""

    total: str
    """Total session cost as a decimal string."""


class Meta(BaseModel):
    event_count: int
    """Total number of events in the session tree."""

    products: List[str]
    """List of distinct products involved in the session."""


class SessionAnalysisRetrieveResponse(BaseModel):
    cost: Cost

    created_at: datetime
    """When the session started."""

    meta: Meta

    root: "EventNode"

    session_id: str
    """Identifier for the analyzed session."""

    status: str
    """Analysis status (e.g. "completed")."""

    completed_at: Optional[datetime] = None
    """When the session completed."""


from .event_node import EventNode



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List
from typing_extensions import Literal, TypedDict

__all__ = ["AvailablePhoneNumberListParams", "Filter", "FilterPhoneNumber"]


class AvailablePhoneNumberListParams(TypedDict, total=False):
    filter: Filter
    """Consolidated filter parameter (deepObject style).

    Originally: filter[phone_number], filter[locality], filter[administrative_area],
    filter[country_code], filter[national_destination_code], filter[rate_center],
    filter[phone_number_type], filter[features], filter[limit], filter[best_effort],
    filter[quickship], filter[reservable], filter[exclude_held_numbers]
    """


class FilterPhoneNumber(TypedDict, total=False):
    """Filter phone numbers by pattern matching."""

    contains: str
    """
    Filter numbers containing a pattern (excludes NDC if used with
    `national_destination_code` filter).
    """

    ends_with: str
    """
    Filter numbers ending with a pattern (excludes NDC if used with
    `national_destination_code` filter).
    """

    starts_with: str
    """
    Filter numbers starting with a pattern (excludes NDC if used with
    `national_destination_code` filter).
    """


class Filter(TypedDict, total=False):
    """Consolidated filter parameter (deepObject style).

    Originally: filter[phone_number], filter[locality], filter[administrative_area], filter[country_code], filter[national_destination_code], filter[rate_center], filter[phone_number_type], filter[features], filter[limit], filter[best_effort], filter[quickship], filter[reservable], filter[exclude_held_numbers]
    """

    administrative_area: str
    """Find numbers in a particular US state or CA province."""

    best_effort: bool
    """Filter to determine if best effort results should be included.

    Only available in USA/CANADA.
    """

    country_code: str
    """Filter phone numbers by country."""

    exclude_held_numbers: bool
    """
    Filter to exclude phone numbers that are currently on hold/reserved for your
    account.
    """

    features: List[Literal["sms", "mms", "voice", "fax", "emergency", "hd_voice", "international_sms", "local_calling"]]
    """Filter phone numbers with specific features."""

    limit: int
    """Limits the number of results."""

    locality: str
    """Filter phone numbers by city."""

    national_destination_code: str
    """Filter by the national destination code of the number."""

    phone_number: FilterPhoneNumber
    """Filter phone numbers by pattern matching."""

    phone_number_type: Literal["local", "toll_free", "mobile", "national", "shared_cost"]
    """Filter phone numbers by number type."""

    quickship: bool
    """
    Filter to exclude phone numbers that need additional time after to purchase to
    activate. Only applicable for +1 toll_free numbers.
    """

    rate_center: str
    """Filter phone numbers by rate center.

    This filter is only applicable to USA and Canada numbers.
    """

    reservable: bool
    """Filter to ensure only numbers that can be reserved are included in the results."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import httpx

from .._types import Body, Query, Headers, NotGiven, not_given
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from .._base_client import make_request_options
from ..types.well_known_retrieve_protected_resource_metadata_response import (
    WellKnownRetrieveProtectedResourceMetadataResponse,
)
from ..types.well_known_retrieve_authorization_server_metadata_response import (
    WellKnownRetrieveAuthorizationServerMetadataResponse,
)

__all__ = ["WellKnownResource", "AsyncWellKnownResource"]


class WellKnownResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> WellKnownResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return WellKnownResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> WellKnownResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return WellKnownResourceWithStreamingResponse(self)

    def retrieve_authorization_server_metadata(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> WellKnownRetrieveAuthorizationServerMetadataResponse:
        """OAuth 2.0 Authorization Server Metadata (RFC 8414)"""
        return self._get(
            "/.well-known/oauth-authorization-server"
            if self._client._base_url_overridden
            else "https://api.telnyx.com/.well-known/oauth-authorization-server",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=WellKnownRetrieveAuthorizationServerMetadataResponse,
        )

    def retrieve_protected_resource_metadata(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> WellKnownRetrieveProtectedResourceMetadataResponse:
        """OAuth 2.0 Protected Resource Metadata for resource discovery"""
        return self._get(
            "/.well-known/oauth-protected-resource"
            if self._client._base_url_overridden
            else "https://api.telnyx.com/.well-known/oauth-protected-resource",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=WellKnownRetrieveProtectedResourceMetadataResponse,
        )


class AsyncWellKnownResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncWellKnownResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return AsyncWellKnownResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncWellKnownResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return AsyncWellKnownResourceWithStreamingResponse(self)

    async def retrieve_authorization_server_metadata(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> WellKnownRetrieveAuthorizationServerMetadataResponse:
        """OAuth 2.0 Authorization Server Metadata (RFC 8414)"""
        return await self._get(
            "/.well-known/oauth-authorization-server"
            if self._client._base_url_overridden
            else "https://api.telnyx.com/.well-known/oauth-authorization-server",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=WellKnownRetrieveAuthorizationServerMetadataResponse,
        )

    async def retrieve_protected_resource_metadata(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> WellKnownRetrieveProtectedResourceMetadataResponse:
        """OAuth 2.0 Protected Resource Metadata for resource discovery"""
        return await self._get(
            "/.well-known/oauth-protected-resource"
            if self._client._base_url_overridden
            else "https://api.telnyx.com/.well-known/oauth-protected-resource",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=WellKnownRetrieveProtectedResourceMetadataResponse,
        )


class WellKnownResourceWithRawResponse:
    def __init__(self, well_known: WellKnownResource) -> None:
        self._well_known = well_known

        self.retrieve_authorization_server_metadata = to_raw_response_wrapper(
            well_known.retrieve_authorization_server_metadata,
        )
        self.retrieve_protected_resource_metadata = to_raw_response_wrapper(
            well_known.retrieve_protected_resource_metadata,
        )


class AsyncWellKnownResourceWithRawResponse:
    def __init__(self, well_known: AsyncWellKnownResource) -> None:
        self._well_known = well_known

        self.retrieve_authorization_server_metadata = async_to_raw_response_wrapper(
            well_known.retrieve_authorization_server_metadata,
        )
        self.retrieve_protected_resource_metadata = async_to_raw_response_wrapper(
            well_known.retrieve_protected_resource_metadata,
        )


class WellKnownResourceWithStreamingResponse:
    def __init__(self, well_known: WellKnownResource) -> None:
        self._well_known = well_known

        self.retrieve_authorization_server_metadata = to_streamed_response_wrapper(
            well_known.retrieve_authorization_server_metadata,
        )
        self.retrieve_protected_resource_metadata = to_streamed_response_wrapper(
            well_known.retrieve_protected_resource_metadata,
        )


class AsyncWellKnownResourceWithStreamingResponse:
    def __init__(self, well_known: AsyncWellKnownResource) -> None:
        self._well_known = well_known

        self.retrieve_authorization_server_metadata = async_to_streamed_response_wrapper(
            well_known.retrieve_authorization_server_metadata,
        )
        self.retrieve_protected_resource_metadata = async_to_streamed_response_wrapper(
            well_known.retrieve_protected_resource_metadata,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal

import httpx

from ..types import audit_event_list_params
from .._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from .._utils import maybe_transform
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..pagination import SyncDefaultFlatPagination, AsyncDefaultFlatPagination
from .._base_client import AsyncPaginator, make_request_options
from ..types.audit_event_list_response import AuditEventListResponse

__all__ = ["AuditEventsResource", "AsyncAuditEventsResource"]


class AuditEventsResource(SyncAPIResource):
    """Audit log operations."""

    @cached_property
    def with_raw_response(self) -> AuditEventsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return AuditEventsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AuditEventsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return AuditEventsResourceWithStreamingResponse(self)

    def list(
        self,
        *,
        filter: audit_event_list_params.Filter | Omit = omit,
        page_number: int | Omit = omit,
        page_size: int | Omit = omit,
        sort: Literal["asc", "desc"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncDefaultFlatPagination[AuditEventListResponse]:
        """Retrieve a list of audit log entries.

        Audit logs are a best-effort, eventually
        consistent record of significant account-related changes.

        Args:
          filter:
              Consolidated filter parameter (deepObject style). Originally:
              filter[created_before], filter[created_after]

          sort: Set the order of the results by the creation date.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/audit_events",
            page=SyncDefaultFlatPagination[AuditEventListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "filter": filter,
                        "page_number": page_number,
                        "page_size": page_size,
                        "sort": sort,
                    },
                    audit_event_list_params.AuditEventListParams,
                ),
            ),
            model=AuditEventListResponse,
        )


class AsyncAuditEventsResource(AsyncAPIResource):
    """Audit log operations."""

    @cached_property
    def with_raw_response(self) -> AsyncAuditEventsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return AsyncAuditEventsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncAuditEventsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return AsyncAuditEventsResourceWithStreamingResponse(self)

    def list(
        self,
        *,
        filter: audit_event_list_params.Filter | Omit = omit,
        page_number: int | Omit = omit,
        page_size: int | Omit = omit,
        sort: Literal["asc", "desc"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[AuditEventListResponse, AsyncDefaultFlatPagination[AuditEventListResponse]]:
        """Retrieve a list of audit log entries.

        Audit logs are a best-effort, eventually
        consistent record of significant account-related changes.

        Args:
          filter:
              Consolidated filter parameter (deepObject style). Originally:
              filter[created_before], filter[created_after]

          sort: Set the order of the results by the creation date.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/audit_events",
            page=AsyncDefaultFlatPagination[AuditEventListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "filter": filter,
                        "page_number": page_number,
                        "page_size": page_size,
                        "sort": sort,
                    },
                    audit_event_list_params.AuditEventListParams,
                ),
            ),
            model=AuditEventListResponse,
        )


class AuditEventsResourceWithRawResponse:
    def __init__(self, audit_events: AuditEventsResource) -> None:
        self._audit_events = audit_events

        self.list = to_raw_response_wrapper(
            audit_events.list,
        )


class AsyncAuditEventsResourceWithRawResponse:
    def __init__(self, audit_events: AsyncAuditEventsResource) -> None:
        self._audit_events = audit_events

        self.list = async_to_raw_response_wrapper(
            audit_events.list,
        )


class AuditEventsResourceWithStreamingResponse:
    def __init__(self, audit_events: AuditEventsResource) -> None:
        self._audit_events = audit_events

        self.list = to_streamed_response_wrapper(
            audit_events.list,
        )


class AsyncAuditEventsResourceWithStreamingResponse:
    def __init__(self, audit_events: AsyncAuditEventsResource) -> None:
        self._audit_events = audit_events

        self.list = async_to_streamed_response_wrapper(
            audit_events.list,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional
from typing_extensions import Literal, Required, TypedDict

from .._types import SequenceNotStr
from .anchorsite_override import AnchorsiteOverride

__all__ = ["FaxApplicationCreateParams", "Inbound", "Outbound"]


class FaxApplicationCreateParams(TypedDict, total=False):
    application_name: Required[str]
    """A user-assigned name to help manage the application."""

    webhook_event_url: Required[str]
    """The URL where webhooks related to this connection will be sent.

    Must include a scheme, such as 'https'.
    """

    active: bool
    """Specifies whether the connection can be used."""

    anchorsite_override: AnchorsiteOverride
    """
    `Latency` directs Telnyx to route media through the site with the lowest
    round-trip time to the user's connection. Telnyx calculates this time using ICMP
    ping messages. This can be disabled by specifying a site to handle all media.
    """

    inbound: Inbound

    outbound: Outbound

    tags: SequenceNotStr[str]
    """Tags associated with the Fax Application."""

    webhook_event_failover_url: Optional[str]
    """
    The failover URL where webhooks related to this connection will be sent if
    sending to the primary URL fails. Must include a scheme, such as 'https'.
    """

    webhook_timeout_secs: Optional[int]
    """Specifies how many seconds to wait before timing out a webhook."""


class Inbound(TypedDict, total=False):
    channel_limit: int
    """
    When set, this will limit the number of concurrent inbound calls to phone
    numbers associated with this connection.
    """

    sip_subdomain: str
    """
    Specifies a subdomain that can be used to receive Inbound calls to a Connection,
    in the same way a phone number is used, from a SIP endpoint. Example: the
    subdomain "example.sip.telnyx.com" can be called from any SIP endpoint by using
    the SIP URI "sip:@example.sip.telnyx.com" where the user part can be any
    alphanumeric value. Please note TLS encrypted calls are not allowed for
    subdomain calls.
    """

    sip_subdomain_receive_settings: Literal["only_my_connections", "from_anyone"]
    """
    This option can be enabled to receive calls from: "Anyone" (any SIP endpoint in
    the public Internet) or "Only my connections" (any connection assigned to the
    same Telnyx user).
    """


class Outbound(TypedDict, total=False):
    channel_limit: int
    """
    When set, this will limit the number of concurrent outbound calls to phone
    numbers associated with this connection.
    """

    outbound_voice_profile_id: str
    """Identifies the associated outbound voice profile."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from typing_extensions import Literal

from .._models import BaseModel

__all__ = ["S3ConfigurationData"]


class S3ConfigurationData(BaseModel):
    backend: Literal["s3"]
    """Storage backend type"""

    aws_access_key_id: Optional[str] = None
    """AWS credentials access key id."""

    aws_secret_access_key: Optional[str] = None
    """AWS secret access key."""

    bucket: Optional[str] = None
    """Name of the bucket to be used to store recording files."""

    region: Optional[str] = None
    """Region where the bucket is located."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from telnyx import Telnyx, AsyncTelnyx
from tests.utils import assert_matches_type
from telnyx.types import WebhookDeliveryListResponse, WebhookDeliveryRetrieveResponse
from telnyx.pagination import SyncDefaultFlatPagination, AsyncDefaultFlatPagination

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestWebhookDeliveries:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_retrieve(self, client: Telnyx) -> None:
        webhook_delivery = client.webhook_deliveries.retrieve(
            "C9C0797E-901D-4349-A33C-C2C8F31A92C2",
        )
        assert_matches_type(WebhookDeliveryRetrieveResponse, webhook_delivery, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_retrieve(self, client: Telnyx) -> None:
        response = client.webhook_deliveries.with_raw_response.retrieve(
            "C9C0797E-901D-4349-A33C-C2C8F31A92C2",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        webhook_delivery = response.parse()
        assert_matches_type(WebhookDeliveryRetrieveResponse, webhook_delivery, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_retrieve(self, client: Telnyx) -> None:
        with client.webhook_deliveries.with_streaming_response.retrieve(
            "C9C0797E-901D-4349-A33C-C2C8F31A92C2",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            webhook_delivery = response.parse()
            assert_matches_type(WebhookDeliveryRetrieveResponse, webhook_delivery, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_path_params_retrieve(self, client: Telnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.webhook_deliveries.with_raw_response.retrieve(
                "",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_list(self, client: Telnyx) -> None:
        webhook_delivery = client.webhook_deliveries.list()
        assert_matches_type(SyncDefaultFlatPagination[WebhookDeliveryListResponse], webhook_delivery, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_list_with_all_params(self, client: Telnyx) -> None:
        webhook_delivery = client.webhook_deliveries.list(
            filter={
                "attempts": {"contains": "https://fallback.example.com/webhooks"},
                "event_type": "call_initiated,call.initiated",
                "finished_at": {
                    "gte": "2019-03-29T11:10:00Z",
                    "lte": "2019-03-29T11:10:00Z",
                },
                "started_at": {
                    "gte": "2019-03-29T11:10:00Z",
                    "lte": "2019-03-29T11:10:00Z",
                },
                "status": {"eq": "delivered"},
                "webhook": {"contains": "call.initiated"},
            },
            page_number=0,
            page_size=0,
        )
        assert_matches_type(SyncDefaultFlatPagination[WebhookDeliveryListResponse], webhook_delivery, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_list(self, client: Telnyx) -> None:
        response = client.webhook_deliveries.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        webhook_delivery = response.parse()
        assert_matches_type(SyncDefaultFlatPagination[WebhookDeliveryListResponse], webhook_delivery, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_list(self, client: Telnyx) -> None:
        with client.webhook_deliveries.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            webhook_delivery = response.parse()
            assert_matches_type(
                SyncDefaultFlatPagination[WebhookDeliveryListResponse], webhook_delivery, path=["response"]
            )

        assert cast(Any, response.is_closed) is True


class TestAsyncWebhookDeliveries:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_retrieve(self, async_client: AsyncTelnyx) -> None:
        webhook_delivery = await async_client.webhook_deliveries.retrieve(
            "C9C0797E-901D-4349-A33C-C2C8F31A92C2",
        )
        assert_matches_type(WebhookDeliveryRetrieveResponse, webhook_delivery, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_retrieve(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.webhook_deliveries.with_raw_response.retrieve(
            "C9C0797E-901D-4349-A33C-C2C8F31A92C2",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        webhook_delivery = await response.parse()
        assert_matches_type(WebhookDeliveryRetrieveResponse, webhook_delivery, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_retrieve(self, async_client: AsyncTelnyx) -> None:
        async with async_client.webhook_deliveries.with_streaming_response.retrieve(
            "C9C0797E-901D-4349-A33C-C2C8F31A92C2",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            webhook_delivery = await response.parse()
            assert_matches_type(WebhookDeliveryRetrieveResponse, webhook_delivery, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_path_params_retrieve(self, async_client: AsyncTelnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.webhook_deliveries.with_raw_response.retrieve(
                "",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_list(self, async_client: AsyncTelnyx) -> None:
        webhook_delivery = await async_client.webhook_deliveries.list()
        assert_matches_type(
            AsyncDefaultFlatPagination[WebhookDeliveryListResponse], webhook_delivery, path=["response"]
        )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_list_with_all_params(self, async_client: AsyncTelnyx) -> None:
        webhook_delivery = await async_client.webhook_deliveries.list(
            filter={
                "attempts": {"contains": "https://fallback.example.com/webhooks"},
                "event_type": "call_initiated,call.initiated",
                "finished_at": {
                    "gte": "2019-03-29T11:10:00Z",
                    "lte": "2019-03-29T11:10:00Z",
                },
                "started_at": {
                    "gte": "2019-03-29T11:10:00Z",
                    "lte": "2019-03-29T11:10:00Z",
                },
                "status": {"eq": "delivered"},
                "webhook": {"contains": "call.initiated"},
            },
            page_number=0,
            page_size=0,
        )
        assert_matches_type(
            AsyncDefaultFlatPagination[WebhookDeliveryListResponse], webhook_delivery, path=["response"]
        )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_list(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.webhook_deliveries.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        webhook_delivery = await response.parse()
        assert_matches_type(
            AsyncDefaultFlatPagination[WebhookDeliveryListResponse], webhook_delivery, path=["response"]
        )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncTelnyx) -> None:
        async with async_client.webhook_deliveries.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            webhook_delivery = await response.parse()
            assert_matches_type(
                AsyncDefaultFlatPagination[WebhookDeliveryListResponse], webhook_delivery, path=["response"]
            )

        assert cast(Any, response.is_closed) is True



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from .._models import BaseModel
from .user_address import UserAddress

__all__ = ["UserAddressRetrieveResponse"]


class UserAddressRetrieveResponse(BaseModel):
    data: Optional[UserAddress] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Dict, List, Optional
from datetime import datetime

from ..._models import BaseModel
from .child_relationship_info import ChildRelationshipInfo
from .parent_relationship_info import ParentRelationshipInfo

__all__ = ["MetadataRetrieveResponse", "Meta", "QueryParameters", "RecordType"]


class Meta(BaseModel):
    last_updated: datetime

    total_record_types: int


class QueryParameters(BaseModel):
    default: str

    description: str

    type: str

    enum_values: Optional[List[str]] = None

    max: Optional[int] = None

    min: Optional[int] = None


class RecordType(BaseModel):
    aliases: List[str]

    child_relationships: List[ChildRelationshipInfo]

    description: str

    event: str

    parent_relationships: List[ParentRelationshipInfo]

    product: str

    record_type: str


class MetadataRetrieveResponse(BaseModel):
    meta: Meta

    query_parameters: Dict[str, QueryParameters]
    """Map of supported query parameter names to their definitions."""

    record_types: List[RecordType]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union
from typing_extensions import Literal

import httpx

from ...._types import Body, Omit, Query, Headers, NotGiven, SequenceNotStr, omit, not_given
from ...._utils import maybe_transform, async_maybe_transform
from ...._compat import cached_property
from ...._resource import SyncAPIResource, AsyncAPIResource
from ...._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...._base_client import make_request_options
from ....types.ai.openai import embedding_create_embeddings_params
from ....types.ai.openai.embedding_create_embeddings_response import EmbeddingCreateEmbeddingsResponse
from ....types.ai.openai.embedding_list_embedding_models_response import EmbeddingListEmbeddingModelsResponse

__all__ = ["EmbeddingsResource", "AsyncEmbeddingsResource"]


class EmbeddingsResource(SyncAPIResource):
    """
    OpenAI-compatible embeddings endpoints for generating vector representations of text
    """

    @cached_property
    def with_raw_response(self) -> EmbeddingsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return EmbeddingsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> EmbeddingsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return EmbeddingsResourceWithStreamingResponse(self)

    def create_embeddings(
        self,
        *,
        input: Union[str, SequenceNotStr[str]],
        model: str,
        dimensions: int | Omit = omit,
        encoding_format: Literal["float", "base64"] | Omit = omit,
        user: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> EmbeddingCreateEmbeddingsResponse:
        """Creates an embedding vector representing the input text.

        This endpoint is
        compatible with the
        [OpenAI Embeddings API](https://platform.openai.com/docs/api-reference/embeddings)
        and may be used with the OpenAI JS or Python SDK by setting the base URL to
        `https://api.telnyx.com/v2/ai/openai`.

        Args:
          input: Input text to embed. Can be a string or array of strings.

          model: ID of the model to use. Use the List embedding models endpoint to see available
              models.

          dimensions: The number of dimensions the resulting output embeddings should have. Only
              supported in some models.

          encoding_format: The format to return the embeddings in.

          user: A unique identifier representing your end-user for monitoring and abuse
              detection.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/ai/openai/embeddings",
            body=maybe_transform(
                {
                    "input": input,
                    "model": model,
                    "dimensions": dimensions,
                    "encoding_format": encoding_format,
                    "user": user,
                },
                embedding_create_embeddings_params.EmbeddingCreateEmbeddingsParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=EmbeddingCreateEmbeddingsResponse,
        )

    def list_embedding_models(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> EmbeddingListEmbeddingModelsResponse:
        """Returns a list of available embedding models.

        This endpoint is compatible with
        the OpenAI Models API format.
        """
        return self._get(
            "/ai/openai/embeddings/models",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=EmbeddingListEmbeddingModelsResponse,
        )


class AsyncEmbeddingsResource(AsyncAPIResource):
    """
    OpenAI-compatible embeddings endpoints for generating vector representations of text
    """

    @cached_property
    def with_raw_response(self) -> AsyncEmbeddingsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return AsyncEmbeddingsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncEmbeddingsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return AsyncEmbeddingsResourceWithStreamingResponse(self)

    async def create_embeddings(
        self,
        *,
        input: Union[str, SequenceNotStr[str]],
        model: str,
        dimensions: int | Omit = omit,
        encoding_format: Literal["float", "base64"] | Omit = omit,
        user: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> EmbeddingCreateEmbeddingsResponse:
        """Creates an embedding vector representing the input text.

        This endpoint is
        compatible with the
        [OpenAI Embeddings API](https://platform.openai.com/docs/api-reference/embeddings)
        and may be used with the OpenAI JS or Python SDK by setting the base URL to
        `https://api.telnyx.com/v2/ai/openai`.

        Args:
          input: Input text to embed. Can be a string or array of strings.

          model: ID of the model to use. Use the List embedding models endpoint to see available
              models.

          dimensions: The number of dimensions the resulting output embeddings should have. Only
              supported in some models.

          encoding_format: The format to return the embeddings in.

          user: A unique identifier representing your end-user for monitoring and abuse
              detection.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/ai/openai/embeddings",
            body=await async_maybe_transform(
                {
                    "input": input,
                    "model": model,
                    "dimensions": dimensions,
                    "encoding_format": encoding_format,
                    "user": user,
                },
                embedding_create_embeddings_params.EmbeddingCreateEmbeddingsParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=EmbeddingCreateEmbeddingsResponse,
        )

    async def list_embedding_models(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> EmbeddingListEmbeddingModelsResponse:
        """Returns a list of available embedding models.

        This endpoint is compatible with
        the OpenAI Models API format.
        """
        return await self._get(
            "/ai/openai/embeddings/models",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=EmbeddingListEmbeddingModelsResponse,
        )


class EmbeddingsResourceWithRawResponse:
    def __init__(self, embeddings: EmbeddingsResource) -> None:
        self._embeddings = embeddings

        self.create_embeddings = to_raw_response_wrapper(
            embeddings.create_embeddings,
        )
        self.list_embedding_models = to_raw_response_wrapper(
            embeddings.list_embedding_models,
        )


class AsyncEmbeddingsResourceWithRawResponse:
    def __init__(self, embeddings: AsyncEmbeddingsResource) -> None:
        self._embeddings = embeddings

        self.create_embeddings = async_to_raw_response_wrapper(
            embeddings.create_embeddings,
        )
        self.list_embedding_models = async_to_raw_response_wrapper(
            embeddings.list_embedding_models,
        )


class EmbeddingsResourceWithStreamingResponse:
    def __init__(self, embeddings: EmbeddingsResource) -> None:
        self._embeddings = embeddings

        self.create_embeddings = to_streamed_response_wrapper(
            embeddings.create_embeddings,
        )
        self.list_embedding_models = to_streamed_response_wrapper(
            embeddings.list_embedding_models,
        )


class AsyncEmbeddingsResourceWithStreamingResponse:
    def __init__(self, embeddings: AsyncEmbeddingsResource) -> None:
        self._embeddings = embeddings

        self.create_embeddings = async_to_streamed_response_wrapper(
            embeddings.create_embeddings,
        )
        self.list_embedding_models = async_to_streamed_response_wrapper(
            embeddings.list_embedding_models,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, Union
from typing_extensions import Required, Annotated, TypedDict

from ..._utils import PropertyInfo

__all__ = ["AssistantSendSMSParams"]


class AssistantSendSMSParams(TypedDict, total=False):
    from_: Required[Annotated[str, PropertyInfo(alias="from")]]

    to: Required[str]

    conversation_metadata: Dict[str, Union[str, int, bool]]

    should_create_conversation: bool

    text: str



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from ...._models import BaseModel
from .whatsapp_profile_data import WhatsappProfileData

__all__ = ["ProfileUpdateResponse"]


class ProfileUpdateResponse(BaseModel):
    data: Optional[WhatsappProfileData] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from typing_extensions import Literal

from pydantic import Field as FieldInfo

from .._models import BaseModel

__all__ = ["CampaignStatusUpdate"]


class CampaignStatusUpdate(BaseModel):
    brand_id: Optional[str] = FieldInfo(alias="brandId", default=None)
    """Brand ID associated with the campaign."""

    campaign_id: Optional[str] = FieldInfo(alias="campaignId", default=None)
    """The ID of the campaign."""

    create_date: Optional[str] = FieldInfo(alias="createDate", default=None)
    """Unix timestamp when campaign was created."""

    csp_id: Optional[str] = FieldInfo(alias="cspId", default=None)
    """Alphanumeric identifier of the CSP associated with this campaign."""

    description: Optional[str] = None
    """Description of the event."""

    is_t_mobile_registered: Optional[bool] = FieldInfo(alias="isTMobileRegistered", default=None)
    """Indicates whether the campaign is registered with T-Mobile."""

    status: Optional[Literal["ACCEPTED", "REJECTED", "DORMANT", "success", "failed"]] = None
    """The status of the campaign."""

    type: Optional[
        Literal[
            "TELNYX_EVENT",
            "REGISTRATION",
            "MNO_REVIEW",
            "TELNYX_REVIEW",
            "NUMBER_POOL_PROVISIONED",
            "NUMBER_POOL_DEPROVISIONED",
            "TCR_EVENT",
            "VERIFIED",
        ]
    ] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from ..._models import BaseModel
from .metadata_field_mapping import MetadataFieldMapping

__all__ = ["ParentRelationshipInfo"]


class ParentRelationshipInfo(BaseModel):
    cost_rollup: bool

    description: str

    parent_event: str

    parent_product: str

    parent_record_type: str

    relationship_type: str

    traversal_enabled: bool

    via: MetadataFieldMapping



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from typing_extensions import Literal

from .._models import BaseModel
from .whatsapp_media import WhatsappMedia

__all__ = [
    "WhatsappInteractive",
    "Action",
    "ActionButton",
    "ActionButtonReply",
    "ActionCard",
    "ActionCardAction",
    "ActionCardBody",
    "ActionCardHeader",
    "ActionParameters",
    "ActionSection",
    "ActionSectionProductItem",
    "ActionSectionRow",
    "Body",
    "Footer",
    "Header",
]


class ActionButtonReply(BaseModel):
    id: Optional[str] = None
    """unique identifier for each button, 256 character maximum"""

    title: Optional[str] = None
    """button label, 20 character maximum"""


class ActionButton(BaseModel):
    reply: Optional[ActionButtonReply] = None

    type: Optional[Literal["reply"]] = None


class ActionCardAction(BaseModel):
    catalog_id: Optional[str] = None
    """the unique ID of the catalog"""

    product_retailer_id: Optional[str] = None
    """the unique retailer ID of the product"""


class ActionCardBody(BaseModel):
    text: Optional[str] = None
    """160 character maximum, up to 2 line breaks"""


class ActionCardHeader(BaseModel):
    image: Optional[WhatsappMedia] = None

    type: Optional[Literal["image", "video"]] = None

    video: Optional[WhatsappMedia] = None


class ActionCard(BaseModel):
    action: Optional[ActionCardAction] = None

    body: Optional[ActionCardBody] = None

    card_index: Optional[int] = None
    """unique index for each card (0-9)"""

    header: Optional[ActionCardHeader] = None

    type: Optional[Literal["cta_url"]] = None


class ActionParameters(BaseModel):
    display_text: Optional[str] = None
    """button label text, 20 character maximum"""

    url: Optional[str] = None
    """button URL to load when tapped by the user"""


class ActionSectionProductItem(BaseModel):
    product_retailer_id: Optional[str] = None


class ActionSectionRow(BaseModel):
    id: Optional[str] = None
    """arbitrary string identifying the row, 200 character maximum"""

    description: Optional[str] = None
    """row description, 72 character maximum"""

    title: Optional[str] = None
    """row title, 24 character maximum"""


class ActionSection(BaseModel):
    product_items: Optional[List[ActionSectionProductItem]] = None

    rows: Optional[List[ActionSectionRow]] = None

    title: Optional[str] = None
    """section title, 24 character maximum"""


class Action(BaseModel):
    button: Optional[str] = None

    buttons: Optional[List[ActionButton]] = None

    cards: Optional[List[ActionCard]] = None

    catalog_id: Optional[str] = None

    mode: Optional[str] = None

    name: Optional[str] = None

    parameters: Optional[ActionParameters] = None

    product_retailer_id: Optional[str] = None

    sections: Optional[List[ActionSection]] = None


class Body(BaseModel):
    text: Optional[str] = None
    """body text, 1024 character maximum"""


class Footer(BaseModel):
    text: Optional[str] = None
    """footer text, 60 character maximum"""


class Header(BaseModel):
    document: Optional[WhatsappMedia] = None

    image: Optional[WhatsappMedia] = None

    sub_text: Optional[str] = None

    text: Optional[str] = None
    """header text, 60 character maximum"""

    video: Optional[WhatsappMedia] = None


class WhatsappInteractive(BaseModel):
    action: Optional[Action] = None

    body: Optional[Body] = None

    footer: Optional[Footer] = None

    header: Optional[Header] = None

    type: Optional[Literal["cta_url", "list", "carousel", "button", "location_request_message"]] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union
from datetime import date
from typing_extensions import Literal

import httpx

from ....._types import Body, Omit, Query, Headers, NoneType, NotGiven, SequenceNotStr, omit, not_given
from ....._utils import maybe_transform, async_maybe_transform
from ....._compat import cached_property
from ....._resource import SyncAPIResource, AsyncAPIResource
from ....._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ....._base_client import make_request_options
from .....types.legacy.reporting.usage_reports import number_lookup_create_params
from .....types.legacy.reporting.usage_reports.number_lookup_list_response import NumberLookupListResponse
from .....types.legacy.reporting.usage_reports.number_lookup_create_response import NumberLookupCreateResponse
from .....types.legacy.reporting.usage_reports.number_lookup_retrieve_response import NumberLookupRetrieveResponse

__all__ = ["NumberLookupResource", "AsyncNumberLookupResource"]


class NumberLookupResource(SyncAPIResource):
    """Number lookup usage reports"""

    @cached_property
    def with_raw_response(self) -> NumberLookupResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return NumberLookupResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> NumberLookupResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return NumberLookupResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        aggregation_type: Literal["ALL", "BY_ORGANIZATION_MEMBER"] | Omit = omit,
        end_date: Union[str, date] | Omit = omit,
        managed_accounts: SequenceNotStr[str] | Omit = omit,
        start_date: Union[str, date] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> NumberLookupCreateResponse:
        """
        Submit a new telco data usage report

        Args:
          aggregation_type: Type of aggregation for the report

          end_date: End date for the usage report

          managed_accounts: List of managed accounts to include in the report

          start_date: Start date for the usage report

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/legacy/reporting/usage_reports/number_lookup",
            body=maybe_transform(
                {
                    "aggregation_type": aggregation_type,
                    "end_date": end_date,
                    "managed_accounts": managed_accounts,
                    "start_date": start_date,
                },
                number_lookup_create_params.NumberLookupCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NumberLookupCreateResponse,
        )

    def retrieve(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> NumberLookupRetrieveResponse:
        """
        Retrieve a specific telco data usage report by its ID

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._get(
            f"/legacy/reporting/usage_reports/number_lookup/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NumberLookupRetrieveResponse,
        )

    def list(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> NumberLookupListResponse:
        """Retrieve a paginated list of telco data usage reports"""
        return self._get(
            "/legacy/reporting/usage_reports/number_lookup",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NumberLookupListResponse,
        )

    def delete(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """
        Delete a specific telco data usage report by its ID

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return self._delete(
            f"/legacy/reporting/usage_reports/number_lookup/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )


class AsyncNumberLookupResource(AsyncAPIResource):
    """Number lookup usage reports"""

    @cached_property
    def with_raw_response(self) -> AsyncNumberLookupResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return AsyncNumberLookupResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncNumberLookupResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return AsyncNumberLookupResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        aggregation_type: Literal["ALL", "BY_ORGANIZATION_MEMBER"] | Omit = omit,
        end_date: Union[str, date] | Omit = omit,
        managed_accounts: SequenceNotStr[str] | Omit = omit,
        start_date: Union[str, date] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> NumberLookupCreateResponse:
        """
        Submit a new telco data usage report

        Args:
          aggregation_type: Type of aggregation for the report

          end_date: End date for the usage report

          managed_accounts: List of managed accounts to include in the report

          start_date: Start date for the usage report

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/legacy/reporting/usage_reports/number_lookup",
            body=await async_maybe_transform(
                {
                    "aggregation_type": aggregation_type,
                    "end_date": end_date,
                    "managed_accounts": managed_accounts,
                    "start_date": start_date,
                },
                number_lookup_create_params.NumberLookupCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NumberLookupCreateResponse,
        )

    async def retrieve(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> NumberLookupRetrieveResponse:
        """
        Retrieve a specific telco data usage report by its ID

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._get(
            f"/legacy/reporting/usage_reports/number_lookup/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NumberLookupRetrieveResponse,
        )

    async def list(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> NumberLookupListResponse:
        """Retrieve a paginated list of telco data usage reports"""
        return await self._get(
            "/legacy/reporting/usage_reports/number_lookup",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NumberLookupListResponse,
        )

    async def delete(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """
        Delete a specific telco data usage report by its ID

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return await self._delete(
            f"/legacy/reporting/usage_reports/number_lookup/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )


class NumberLookupResourceWithRawResponse:
    def __init__(self, number_lookup: NumberLookupResource) -> None:
        self._number_lookup = number_lookup

        self.create = to_raw_response_wrapper(
            number_lookup.create,
        )
        self.retrieve = to_raw_response_wrapper(
            number_lookup.retrieve,
        )
        self.list = to_raw_response_wrapper(
            number_lookup.list,
        )
        self.delete = to_raw_response_wrapper(
            number_lookup.delete,
        )


class AsyncNumberLookupResourceWithRawResponse:
    def __init__(self, number_lookup: AsyncNumberLookupResource) -> None:
        self._number_lookup = number_lookup

        self.create = async_to_raw_response_wrapper(
            number_lookup.create,
        )
        self.retrieve = async_to_raw_response_wrapper(
            number_lookup.retrieve,
        )
        self.list = async_to_raw_response_wrapper(
            number_lookup.list,
        )
        self.delete = async_to_raw_response_wrapper(
            number_lookup.delete,
        )


class NumberLookupResourceWithStreamingResponse:
    def __init__(self, number_lookup: NumberLookupResource) -> None:
        self._number_lookup = number_lookup

        self.create = to_streamed_response_wrapper(
            number_lookup.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            number_lookup.retrieve,
        )
        self.list = to_streamed_response_wrapper(
            number_lookup.list,
        )
        self.delete = to_streamed_response_wrapper(
            number_lookup.delete,
        )


class AsyncNumberLookupResourceWithStreamingResponse:
    def __init__(self, number_lookup: AsyncNumberLookupResource) -> None:
        self._number_lookup = number_lookup

        self.create = async_to_streamed_response_wrapper(
            number_lookup.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            number_lookup.retrieve,
        )
        self.list = async_to_streamed_response_wrapper(
            number_lookup.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            number_lookup.delete,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from datetime import datetime

from ..._models import BaseModel

__all__ = ["PortingPhoneNumberExtension", "ActivationRange", "ExtensionRange"]


class ActivationRange(BaseModel):
    end_at: Optional[int] = None
    """Specifies the end of the activation range.

    It must be no more than the end of the extension range.
    """

    start_at: Optional[int] = None
    """Specifies the start of the activation range.

    Must be greater or equal the start of the extension range.
    """


class ExtensionRange(BaseModel):
    """Specifies the extension range for this porting phone number extension."""

    end_at: Optional[int] = None
    """
    Specifies the end of the extension range for this porting phone number
    extension.
    """

    start_at: Optional[int] = None
    """
    Specifies the start of the extension range for this porting phone number
    extension.
    """


class PortingPhoneNumberExtension(BaseModel):
    id: Optional[str] = None
    """Uniquely identifies this porting phone number extension."""

    activation_ranges: Optional[List[ActivationRange]] = None
    """Specifies the activation ranges for this porting phone number extension.

    The activation range must be within the extension range and should not overlap
    with other activation ranges.
    """

    created_at: Optional[datetime] = None
    """ISO 8601 formatted date indicating when the resource was created."""

    extension_range: Optional[ExtensionRange] = None
    """Specifies the extension range for this porting phone number extension."""

    porting_phone_number_id: Optional[str] = None
    """
    Identifies the porting phone number associated with this porting phone number
    extension.
    """

    record_type: Optional[str] = None
    """Identifies the type of the resource."""

    updated_at: Optional[datetime] = None
    """ISO 8601 formatted date indicating when the resource was last updated."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Annotated, TypedDict

from ..._utils import PropertyInfo

__all__ = ["PhoneNumberCampaignListParams", "Filter"]


class PhoneNumberCampaignListParams(TypedDict, total=False):
    filter: Filter
    """Consolidated filter parameter (deepObject style).

    Originally: filter[telnyx_campaign_id], filter[telnyx_brand_id],
    filter[tcr_campaign_id], filter[tcr_brand_id]
    """

    page: int

    records_per_page: Annotated[int, PropertyInfo(alias="recordsPerPage")]

    sort: Literal["assignmentStatus", "-assignmentStatus", "createdAt", "-createdAt", "phoneNumber", "-phoneNumber"]
    """Specifies the sort order for results.

    If not given, results are sorted by createdAt in descending order.
    """


class Filter(TypedDict, total=False):
    """Consolidated filter parameter (deepObject style).

    Originally: filter[telnyx_campaign_id], filter[telnyx_brand_id], filter[tcr_campaign_id], filter[tcr_brand_id]
    """

    tcr_brand_id: str
    """Filter results by the TCR Brand id"""

    tcr_campaign_id: str
    """Filter results by the TCR Campaign id"""

    telnyx_brand_id: str
    """Filter results by the Telnyx Brand id"""

    telnyx_campaign_id: str
    """Filter results by the Telnyx Campaign id"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from .._models import BaseModel
from .voice_design_data import VoiceDesignData

__all__ = ["VoiceDesignRetrieveResponse"]


class VoiceDesignRetrieveResponse(BaseModel):
    """Response envelope for a single voice design with full version detail."""

    data: Optional[VoiceDesignData] = None
    """A voice design object with full version detail."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import TypedDict

from .._types import SequenceNotStr

__all__ = ["PortabilityCheckRunParams"]


class PortabilityCheckRunParams(TypedDict, total=False):
    phone_numbers: SequenceNotStr[str]
    """The list of +E.164 formatted phone numbers to check for portability"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Union, Optional
from typing_extensions import Literal, Annotated, TypeAlias

from ..._utils import PropertyInfo
from ..._models import BaseModel

__all__ = [
    "VoiceSettings",
    "BackgroundAudio",
    "BackgroundAudioPredefinedMedia",
    "BackgroundAudioMediaURL",
    "BackgroundAudioMediaName",
]


class BackgroundAudioPredefinedMedia(BaseModel):
    type: Literal["predefined_media"]
    """Select from predefined media options."""

    value: Literal["silence", "office"]
    """The predefined media to use. `silence` disables background audio."""


class BackgroundAudioMediaURL(BaseModel):
    type: Literal["media_url"]
    """Provide a direct URL to an MP3 file. The audio will loop during the call."""

    value: str
    """HTTPS URL to an MP3 file."""


class BackgroundAudioMediaName(BaseModel):
    type: Literal["media_name"]
    """Reference a previously uploaded media by its name from Telnyx Media Storage."""

    value: str
    """
    The `name` of a media asset created via
    [Media Storage API](https://developers.telnyx.com/api/media-storage/create-media-storage).
    The audio will loop during the call.
    """


BackgroundAudio: TypeAlias = Annotated[
    Union[BackgroundAudioPredefinedMedia, BackgroundAudioMediaURL, BackgroundAudioMediaName],
    PropertyInfo(discriminator="type"),
]


class VoiceSettings(BaseModel):
    voice: str
    """The voice to be used by the voice assistant.

    Check the full list of
    [available voices](https://developers.telnyx.com/api/call-control/list-text-to-speech-voices)
    via our voices API. To use ElevenLabs, you must reference your ElevenLabs API
    key as an integration secret under the `api_key_ref` field. See
    [integration secrets documentation](https://developers.telnyx.com/api-reference/integration-secrets/create-a-secret)
    for details. For Telnyx voices, use `Telnyx.<model_id>.<voice_id>` (e.g.
    Telnyx.KokoroTTS.af_heart)
    """

    api_key_ref: Optional[str] = None
    """
    The `identifier` for an integration secret
    [/v2/integration_secrets](https://developers.telnyx.com/api-reference/integration-secrets/create-a-secret)
    that refers to your ElevenLabs API key. Warning: Free plans are unlikely to work
    with this integration.
    """

    background_audio: Optional[BackgroundAudio] = None
    """Optional background audio to play on the call.

    Use a predefined media bed, or supply a looped MP3 URL. If a media URL is chosen
    in the portal, customers can preview it before saving.
    """

    language_boost: Optional[
        Literal[
            "auto",
            "Chinese",
            "Chinese,Yue",
            "English",
            "Arabic",
            "Russian",
            "Spanish",
            "French",
            "Portuguese",
            "German",
            "Turkish",
            "Dutch",
            "Ukrainian",
            "Vietnamese",
            "Indonesian",
            "Japanese",
            "Italian",
            "Korean",
            "Thai",
            "Polish",
            "Romanian",
            "Greek",
            "Czech",
            "Finnish",
            "Hindi",
            "Bulgarian",
            "Danish",
            "Hebrew",
            "Malay",
            "Persian",
            "Slovak",
            "Swedish",
            "Croatian",
            "Filipino",
            "Hungarian",
            "Norwegian",
            "Slovenian",
            "Catalan",
            "Nynorsk",
            "Tamil",
            "Afrikaans",
        ]
    ] = None
    """
    Enhances recognition for specific languages and dialects during MiniMax TTS
    synthesis. Default is null (no boost). Set to 'auto' for automatic language
    detection. Only applicable when using MiniMax voices.
    """

    similarity_boost: Optional[float] = None
    """
    Determines how closely the AI should adhere to the original voice when
    attempting to replicate it. Only applicable when using ElevenLabs.
    """

    speed: Optional[float] = None
    """Adjusts speech velocity.

    1.0 is default speed; values less than 1.0 slow speech; values greater than 1.0
    accelerate it. Only applicable when using ElevenLabs.
    """

    style: Optional[float] = None
    """Determines the style exaggeration of the voice.

    Amplifies speaker style but consumes additional resources when set above 0. Only
    applicable when using ElevenLabs.
    """

    temperature: Optional[float] = None
    """Determines how stable the voice is and the randomness between each generation.

    Lower values create a broader emotional range; higher values produce more
    consistent, monotonous output. Only applicable when using ElevenLabs.
    """

    use_speaker_boost: Optional[bool] = None
    """Amplifies similarity to the original speaker voice.

    Increases computational load and latency slightly. Only applicable when using
    ElevenLabs.
    """

    voice_speed: Optional[float] = None
    """The speed of the voice in the range [0.25, 2.0].

    1.0 is deafult speed. Larger numbers make the voice faster, smaller numbers make
    it slower. This is only applicable for Telnyx Natural voices.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from ..._models import BaseModel
from ..shared.whatsapp_template_data import WhatsappTemplateData

__all__ = ["TemplateCreateResponse"]


class TemplateCreateResponse(BaseModel):
    data: Optional[WhatsappTemplateData] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

__all__ = ["ChannelZoneUpdateParams"]


class ChannelZoneUpdateParams(TypedDict, total=False):
    channels: Required[int]
    """The number of reserved channels"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal

import httpx

from ....._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ....._utils import maybe_transform, async_maybe_transform
from ....._compat import cached_property
from ....._resource import SyncAPIResource, AsyncAPIResource
from ....._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ....._base_client import make_request_options
from .....types.texml.accounts.calls import recordings_json_recordings_json_params
from .....types.texml.accounts.calls.recordings_json_recordings_json_response import (
    RecordingsJsonRecordingsJsonResponse,
)
from .....types.texml.accounts.calls.recordings_json_retrieve_recordings_json_response import (
    RecordingsJsonRetrieveRecordingsJsonResponse,
)

__all__ = ["RecordingsJsonResource", "AsyncRecordingsJsonResource"]


class RecordingsJsonResource(SyncAPIResource):
    """TeXML REST Commands"""

    @cached_property
    def with_raw_response(self) -> RecordingsJsonResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return RecordingsJsonResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> RecordingsJsonResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return RecordingsJsonResourceWithStreamingResponse(self)

    def recordings_json(
        self,
        call_sid: str,
        *,
        account_sid: str,
        play_beep: bool | Omit = omit,
        recording_channels: Literal["single", "dual"] | Omit = omit,
        recording_status_callback: str | Omit = omit,
        recording_status_callback_event: str | Omit = omit,
        recording_status_callback_method: Literal["GET", "POST"] | Omit = omit,
        recording_track: Literal["inbound", "outbound", "both"] | Omit = omit,
        send_recording_url: bool | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> RecordingsJsonRecordingsJsonResponse:
        """
        Starts recording with specified parameters for call idientified by call_sid.

        Args:
          play_beep: Whether to play a beep when recording is started.

          recording_channels: When `dual`, final audio file has the first leg on channel A, and the rest on
              channel B. `single` mixes both tracks into a single channel.

          recording_status_callback: Url where status callbacks will be sent.

          recording_status_callback_event: The changes to the recording's state that should generate a call to
              `RecoridngStatusCallback`. Can be: `in-progress`, `completed` and `absent`.
              Separate multiple values with a space. Defaults to `completed`.

          recording_status_callback_method: HTTP method used to send status callbacks.

          recording_track: The audio track to record for the call. The default is `both`.

          send_recording_url: Whether to send RecordingUrl in webhooks.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_sid:
            raise ValueError(f"Expected a non-empty value for `account_sid` but received {account_sid!r}")
        if not call_sid:
            raise ValueError(f"Expected a non-empty value for `call_sid` but received {call_sid!r}")
        return self._post(
            f"/texml/Accounts/{account_sid}/Calls/{call_sid}/Recordings.json",
            body=maybe_transform(
                {
                    "play_beep": play_beep,
                    "recording_channels": recording_channels,
                    "recording_status_callback": recording_status_callback,
                    "recording_status_callback_event": recording_status_callback_event,
                    "recording_status_callback_method": recording_status_callback_method,
                    "recording_track": recording_track,
                    "send_recording_url": send_recording_url,
                },
                recordings_json_recordings_json_params.RecordingsJsonRecordingsJsonParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=RecordingsJsonRecordingsJsonResponse,
        )

    def retrieve_recordings_json(
        self,
        call_sid: str,
        *,
        account_sid: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> RecordingsJsonRetrieveRecordingsJsonResponse:
        """
        Returns recordings for a call identified by call_sid.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_sid:
            raise ValueError(f"Expected a non-empty value for `account_sid` but received {account_sid!r}")
        if not call_sid:
            raise ValueError(f"Expected a non-empty value for `call_sid` but received {call_sid!r}")
        return self._get(
            f"/texml/Accounts/{account_sid}/Calls/{call_sid}/Recordings.json",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=RecordingsJsonRetrieveRecordingsJsonResponse,
        )


class AsyncRecordingsJsonResource(AsyncAPIResource):
    """TeXML REST Commands"""

    @cached_property
    def with_raw_response(self) -> AsyncRecordingsJsonResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return AsyncRecordingsJsonResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncRecordingsJsonResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return AsyncRecordingsJsonResourceWithStreamingResponse(self)

    async def recordings_json(
        self,
        call_sid: str,
        *,
        account_sid: str,
        play_beep: bool | Omit = omit,
        recording_channels: Literal["single", "dual"] | Omit = omit,
        recording_status_callback: str | Omit = omit,
        recording_status_callback_event: str | Omit = omit,
        recording_status_callback_method: Literal["GET", "POST"] | Omit = omit,
        recording_track: Literal["inbound", "outbound", "both"] | Omit = omit,
        send_recording_url: bool | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> RecordingsJsonRecordingsJsonResponse:
        """
        Starts recording with specified parameters for call idientified by call_sid.

        Args:
          play_beep: Whether to play a beep when recording is started.

          recording_channels: When `dual`, final audio file has the first leg on channel A, and the rest on
              channel B. `single` mixes both tracks into a single channel.

          recording_status_callback: Url where status callbacks will be sent.

          recording_status_callback_event: The changes to the recording's state that should generate a call to
              `RecoridngStatusCallback`. Can be: `in-progress`, `completed` and `absent`.
              Separate multiple values with a space. Defaults to `completed`.

          recording_status_callback_method: HTTP method used to send status callbacks.

          recording_track: The audio track to record for the call. The default is `both`.

          send_recording_url: Whether to send RecordingUrl in webhooks.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_sid:
            raise ValueError(f"Expected a non-empty value for `account_sid` but received {account_sid!r}")
        if not call_sid:
            raise ValueError(f"Expected a non-empty value for `call_sid` but received {call_sid!r}")
        return await self._post(
            f"/texml/Accounts/{account_sid}/Calls/{call_sid}/Recordings.json",
            body=await async_maybe_transform(
                {
                    "play_beep": play_beep,
                    "recording_channels": recording_channels,
                    "recording_status_callback": recording_status_callback,
                    "recording_status_callback_event": recording_status_callback_event,
                    "recording_status_callback_method": recording_status_callback_method,
                    "recording_track": recording_track,
                    "send_recording_url": send_recording_url,
                },
                recordings_json_recordings_json_params.RecordingsJsonRecordingsJsonParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=RecordingsJsonRecordingsJsonResponse,
        )

    async def retrieve_recordings_json(
        self,
        call_sid: str,
        *,
        account_sid: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> RecordingsJsonRetrieveRecordingsJsonResponse:
        """
        Returns recordings for a call identified by call_sid.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_sid:
            raise ValueError(f"Expected a non-empty value for `account_sid` but received {account_sid!r}")
        if not call_sid:
            raise ValueError(f"Expected a non-empty value for `call_sid` but received {call_sid!r}")
        return await self._get(
            f"/texml/Accounts/{account_sid}/Calls/{call_sid}/Recordings.json",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=RecordingsJsonRetrieveRecordingsJsonResponse,
        )


class RecordingsJsonResourceWithRawResponse:
    def __init__(self, recordings_json: RecordingsJsonResource) -> None:
        self._recordings_json = recordings_json

        self.recordings_json = to_raw_response_wrapper(
            recordings_json.recordings_json,
        )
        self.retrieve_recordings_json = to_raw_response_wrapper(
            recordings_json.retrieve_recordings_json,
        )


class AsyncRecordingsJsonResourceWithRawResponse:
    def __init__(self, recordings_json: AsyncRecordingsJsonResource) -> None:
        self._recordings_json = recordings_json

        self.recordings_json = async_to_raw_response_wrapper(
            recordings_json.recordings_json,
        )
        self.retrieve_recordings_json = async_to_raw_response_wrapper(
            recordings_json.retrieve_recordings_json,
        )


class RecordingsJsonResourceWithStreamingResponse:
    def __init__(self, recordings_json: RecordingsJsonResource) -> None:
        self._recordings_json = recordings_json

        self.recordings_json = to_streamed_response_wrapper(
            recordings_json.recordings_json,
        )
        self.retrieve_recordings_json = to_streamed_response_wrapper(
            recordings_json.retrieve_recordings_json,
        )


class AsyncRecordingsJsonResourceWithStreamingResponse:
    def __init__(self, recordings_json: AsyncRecordingsJsonResource) -> None:
        self._recordings_json = recordings_json

        self.recordings_json = async_to_streamed_response_wrapper(
            recordings_json.recordings_json,
        )
        self.retrieve_recordings_json = async_to_streamed_response_wrapper(
            recordings_json.retrieve_recordings_json,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from .._models import BaseModel
from .call_answered import CallAnswered

__all__ = ["CallAnsweredWebhookEvent"]


class CallAnsweredWebhookEvent(BaseModel):
    data: Optional[CallAnswered] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from typing_extensions import Literal

from ..._models import BaseModel

__all__ = ["CallRecording"]


class CallRecording(BaseModel):
    """The call recording settings for a phone number."""

    inbound_call_recording_channels: Optional[Literal["single", "dual"]] = None
    """
    When using 'dual' channels, final audio file will be stereo recorded with the
    first leg on channel A, and the rest on channel B.
    """

    inbound_call_recording_enabled: Optional[bool] = None
    """When enabled, any inbound call to this number will be recorded."""

    inbound_call_recording_format: Optional[Literal["wav", "mp3"]] = None
    """The audio file format for calls being recorded."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List

from ...._models import BaseModel
from .integration_connection import IntegrationConnection

__all__ = ["ConnectionListResponse"]


class ConnectionListResponse(BaseModel):
    data: List[IntegrationConnection]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .conferences import (
    ConferencesResource,
    AsyncConferencesResource,
    ConferencesResourceWithRawResponse,
    AsyncConferencesResourceWithRawResponse,
    ConferencesResourceWithStreamingResponse,
    AsyncConferencesResourceWithStreamingResponse,
)
from .participants import (
    ParticipantsResource,
    AsyncParticipantsResource,
    ParticipantsResourceWithRawResponse,
    AsyncParticipantsResourceWithRawResponse,
    ParticipantsResourceWithStreamingResponse,
    AsyncParticipantsResourceWithStreamingResponse,
)

__all__ = [
    "ParticipantsResource",
    "AsyncParticipantsResource",
    "ParticipantsResourceWithRawResponse",
    "AsyncParticipantsResourceWithRawResponse",
    "ParticipantsResourceWithStreamingResponse",
    "AsyncParticipantsResourceWithStreamingResponse",
    "ConferencesResource",
    "AsyncConferencesResource",
    "ConferencesResourceWithRawResponse",
    "AsyncConferencesResourceWithRawResponse",
    "ConferencesResourceWithStreamingResponse",
    "AsyncConferencesResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Union
from typing_extensions import Annotated, TypeAlias

from ...._utils import PropertyInfo
from .scheduled_sms_event_response import ScheduledSMSEventResponse
from .scheduled_phone_call_event_response import ScheduledPhoneCallEventResponse

__all__ = ["ScheduledEventListResponse"]

ScheduledEventListResponse: TypeAlias = Annotated[
    Union[ScheduledPhoneCallEventResponse, ScheduledSMSEventResponse],
    PropertyInfo(discriminator="telnyx_conversation_channel"),
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from .._models import BaseModel
from .conference_created import ConferenceCreated

__all__ = ["ConferenceCreatedWebhookEvent"]


class ConferenceCreatedWebhookEvent(BaseModel):
    data: Optional[ConferenceCreated] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .whatsapp import (
    WhatsappResource,
    AsyncWhatsappResource,
    WhatsappResourceWithRawResponse,
    AsyncWhatsappResourceWithRawResponse,
    WhatsappResourceWithStreamingResponse,
    AsyncWhatsappResourceWithStreamingResponse,
)
from .templates import (
    TemplatesResource,
    AsyncTemplatesResource,
    TemplatesResourceWithRawResponse,
    AsyncTemplatesResourceWithRawResponse,
    TemplatesResourceWithStreamingResponse,
    AsyncTemplatesResourceWithStreamingResponse,
)
from .phone_numbers import (
    PhoneNumbersResource,
    AsyncPhoneNumbersResource,
    PhoneNumbersResourceWithRawResponse,
    AsyncPhoneNumbersResourceWithRawResponse,
    PhoneNumbersResourceWithStreamingResponse,
    AsyncPhoneNumbersResourceWithStreamingResponse,
)
from .business_accounts import (
    BusinessAccountsResource,
    AsyncBusinessAccountsResource,
    BusinessAccountsResourceWithRawResponse,
    AsyncBusinessAccountsResourceWithRawResponse,
    BusinessAccountsResourceWithStreamingResponse,
    AsyncBusinessAccountsResourceWithStreamingResponse,
)

__all__ = [
    "BusinessAccountsResource",
    "AsyncBusinessAccountsResource",
    "BusinessAccountsResourceWithRawResponse",
    "AsyncBusinessAccountsResourceWithRawResponse",
    "BusinessAccountsResourceWithStreamingResponse",
    "AsyncBusinessAccountsResourceWithStreamingResponse",
    "TemplatesResource",
    "AsyncTemplatesResource",
    "TemplatesResourceWithRawResponse",
    "AsyncTemplatesResourceWithRawResponse",
    "TemplatesResourceWithStreamingResponse",
    "AsyncTemplatesResourceWithStreamingResponse",
    "PhoneNumbersResource",
    "AsyncPhoneNumbersResource",
    "PhoneNumbersResourceWithRawResponse",
    "AsyncPhoneNumbersResourceWithRawResponse",
    "PhoneNumbersResourceWithStreamingResponse",
    "AsyncPhoneNumbersResourceWithStreamingResponse",
    "WhatsappResource",
    "AsyncWhatsappResource",
    "WhatsappResourceWithRawResponse",
    "AsyncWhatsappResourceWithRawResponse",
    "WhatsappResourceWithStreamingResponse",
    "AsyncWhatsappResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing_extensions import Literal, TypeAlias

__all__ = ["BackgroundTaskStatus"]

BackgroundTaskStatus: TypeAlias = Literal["queued", "processing", "success", "failure", "partial_success"]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, Annotated, TypedDict

from ....._utils import PropertyInfo

__all__ = ["RecordingsJsonRecordingsJsonParams"]


class RecordingsJsonRecordingsJsonParams(TypedDict, total=False):
    account_sid: Required[str]

    play_beep: Annotated[bool, PropertyInfo(alias="PlayBeep")]
    """Whether to play a beep when recording is started."""

    recording_channels: Annotated[Literal["single", "dual"], PropertyInfo(alias="RecordingChannels")]
    """
    When `dual`, final audio file has the first leg on channel A, and the rest on
    channel B. `single` mixes both tracks into a single channel.
    """

    recording_status_callback: Annotated[str, PropertyInfo(alias="RecordingStatusCallback")]
    """Url where status callbacks will be sent."""

    recording_status_callback_event: Annotated[str, PropertyInfo(alias="RecordingStatusCallbackEvent")]
    """
    The changes to the recording's state that should generate a call to
    `RecoridngStatusCallback`. Can be: `in-progress`, `completed` and `absent`.
    Separate multiple values with a space. Defaults to `completed`.
    """

    recording_status_callback_method: Annotated[
        Literal["GET", "POST"], PropertyInfo(alias="RecordingStatusCallbackMethod")
    ]
    """HTTP method used to send status callbacks."""

    recording_track: Annotated[Literal["inbound", "outbound", "both"], PropertyInfo(alias="RecordingTrack")]
    """The audio track to record for the call. The default is `both`."""

    send_recording_url: Annotated[bool, PropertyInfo(alias="SendRecordingUrl")]
    """Whether to send RecordingUrl in webhooks."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from .._models import BaseModel
from .voice_clone_data import VoiceCloneData

__all__ = ["VoiceCloneUpdateResponse"]


class VoiceCloneUpdateResponse(BaseModel):
    """Response envelope for a single voice clone."""

    data: Optional[VoiceCloneData] = None
    """A voice clone object."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, TypedDict

__all__ = ["PhoneNumberResendVerificationParams"]


class PhoneNumberResendVerificationParams(TypedDict, total=False):
    verification_method: Literal["sms", "voice"]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, TypedDict

__all__ = ["ActionStartNoiseSuppressionParams", "NoiseSuppressionEngineConfig"]


class ActionStartNoiseSuppressionParams(TypedDict, total=False):
    client_state: str
    """Use this field to add state to every subsequent webhook.

    It must be a valid Base-64 encoded string.
    """

    command_id: str
    """Use this field to avoid duplicate commands.

    Telnyx will ignore any command with the same `command_id` for the same
    `call_control_id`.
    """

    direction: Literal["inbound", "outbound", "both"]
    """The direction of the audio stream to be noise suppressed."""

    noise_suppression_engine: Literal["Denoiser", "DeepFilterNet", "Krisp"]
    """
    The engine to use for noise suppression. For backward compatibility, engines A,
    B, and C are also supported, but are deprecated: A - Denoiser B - DeepFilterNet
    C - Krisp
    """

    noise_suppression_engine_config: NoiseSuppressionEngineConfig
    """Configuration parameters for noise suppression engines."""


class NoiseSuppressionEngineConfig(TypedDict, total=False):
    """Configuration parameters for noise suppression engines."""

    attenuation_limit: int
    """The attenuation limit for noise suppression (0-100).

    Only applicable for DeepFilterNet.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from typing_extensions import Literal

from ..._models import BaseModel

__all__ = ["SimCardStatus"]


class SimCardStatus(BaseModel):
    reason: Optional[str] = None
    """It describes why the SIM card is in the current status."""

    value: Optional[
        Literal[
            "registering",
            "enabling",
            "enabled",
            "disabling",
            "disabled",
            "data_limit_exceeded",
            "setting_standby",
            "standby",
        ]
    ] = None
    """The current status of the SIM card. It will be one of the following: <br/>

    <ul>
     <li><code>registering</code> - the card is being registered</li>
     <li><code>enabling</code> - the card is being enabled</li>
     <li><code>enabled</code> - the card is enabled and ready for use</li>
     <li><code>disabling</code> - the card is being disabled</li>
     <li><code>disabled</code> - the card has been disabled and cannot be used</li>
     <li><code>data_limit_exceeded</code> - the card has exceeded its data consumption limit</li>
     <li><code>setting_standby</code> - the process to set the card in stand by is in progress</li>
     <li><code>standby</code> - the card is in stand by</li>
    </ul>
    Transitioning between the enabled and disabled states may take a period of time.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Dict, Optional

from .._models import BaseModel

__all__ = ["StreamClientEvent"]


class StreamClientEvent(BaseModel):
    """Client-to-server frame containing text to synthesize."""

    text: str
    """Text to convert to speech.

    Send `" "` (single space) as an initial handshake with optional
    `voice_settings`. Subsequent messages contain the actual text to synthesize.
    """

    force: Optional[bool] = None
    """When `true`, stops the current synthesis worker and starts a new one.

    Used to interrupt speech mid-stream and begin synthesizing new text.
    """

    voice_settings: Optional[Dict[str, object]] = None
    """Provider-specific voice settings sent with the initial handshake.

    Contents vary by provider — e.g. `{"speed": 1.2}` for Minimax,
    `{"voice_speed": 1.5}` for Telnyx.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from typing_extensions import Literal

from ..._models import BaseModel

__all__ = ["ImportMetadata"]


class ImportMetadata(BaseModel):
    import_id: Optional[str] = None
    """ID of the assistant in the provider's system."""

    import_provider: Optional[Literal["elevenlabs", "vapi", "retell"]] = None
    """Provider the assistant was imported from."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .photo_upload_params import PhotoUploadParams as PhotoUploadParams
from .photo_upload_response import PhotoUploadResponse as PhotoUploadResponse
